@@ -123,7 +123,7 @@ Event::Event(
     video_incomplete_file = "incomplete."+container;
   }
 
-  std::string now_str = SystemTimePointToMysqlString(now);
+  std::string now_str = SystemTimePointToMysqlString(start_time);
 
   std::string sql = stringtf(
                       "INSERT INTO `Events` "
@@ -205,7 +205,7 @@ Event::~Event() {
     videoStore = nullptr;
     int result = rename(video_incomplete_path.c_str(), video_path.c_str());
     if (result != 0) {
-      Error("Failed renaming %s to %s, reason: %s", video_incomplete_path.c_str(), video_path.c_str(), strerror(result));
+      Error("Failed renaming %s to %s, reason: %s", video_incomplete_path.c_str(), video_path.c_str(), strerror(errno));
       // So that we don't update the event record
       video_file = video_incomplete_file;
     }
@@ -227,43 +227,40 @@ Event::~Event() {
   uint64_t video_size = 0;
   DIR *video_dir;
   if ((video_dir = opendir(path.c_str())) != NULL) {
-    struct dirent *video_file;
-    while ((video_file = readdir(video_dir)) != NULL) {
+    struct dirent *dir_entry;
+    while ((dir_entry = readdir(video_dir)) != NULL) {
       struct stat vf_stat;
-      if (stat((path + "/" + video_file->d_name).c_str(), &vf_stat) == 0 &&
+      if (stat((path + "/" + dir_entry->d_name).c_str(), &vf_stat) == 0 &&
           S_ISREG(vf_stat.st_mode))
         video_size += vf_stat.st_size;
     }
     closedir(video_dir);
   }
 
+  // Use async dbQueue instead of synchronous zmDbDoUpdate to avoid blocking
+  // the close_event_thread (which blocks the analysis thread on the next closeEvent).
+  // Conditionally update Name only if it hasn't been changed by the user during recording.
   std::string sql = stringtf(
-      "UPDATE Events SET Name='%s%" PRIu64 "', EndDateTime = from_unixtime(%jd), Length = %.2f, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d, MaxScoreFrameId=%d, DefaultVideo='%s', DiskSpace=%" PRIu64 " WHERE Id = %" PRIu64 " AND Name='New Event'",
-      monitor->Substitute(monitor->EventPrefix(), start_time).c_str(), id, static_cast<intmax_t>(std::chrono::system_clock::to_time_t(end_time)),
+      "UPDATE Events SET"
+      " Name = IF(Name='New Event', '%s%" PRIu64 "', Name),"
+      " EndDateTime = from_unixtime(%jd), Length = %.2f, Frames = %d,"
+      " AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d,"
+      " MaxScoreFrameId=%d, DefaultVideo='%s', DiskSpace=%" PRIu64
+      " WHERE Id = %" PRIu64,
+      monitor->Substitute(monitor->EventPrefix(), start_time).c_str(), id,
+      static_cast<intmax_t>(std::chrono::system_clock::to_time_t(end_time)),
       delta_time.count(),
       frames, alarm_frames,
-      tot_score, static_cast<uint32>(alarm_frames ? (tot_score / alarm_frames) : 0), max_score, max_score_frame_id,
+      tot_score, static_cast<uint32>(alarm_frames ? (tot_score / alarm_frames) : 0),
+      max_score, max_score_frame_id,
       video_file.c_str(), // defaults to ""
       video_size,
       id);
-
-  if (!zmDbDoUpdate(sql)) {
-    // Name might have been changed during recording, so just do the update without changing the name.
-    sql = stringtf(
-        "UPDATE Events SET EndDateTime = from_unixtime(%jd), Length = %.2f, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d, MaxScoreFrameId=%d, DefaultVideo='%s', DiskSpace=%" PRIu64 " WHERE Id = %" PRIu64,
-        static_cast<intmax_t>(std::chrono::system_clock::to_time_t(end_time)),
-        delta_time.count(),
-        frames, alarm_frames,
-        tot_score, static_cast<uint32>(alarm_frames ? (tot_score / alarm_frames) : 0), max_score, max_score_frame_id,
-        video_file.c_str(), // defaults to ""
-        video_size,
-        id);
-    zmDbDoUpdate(sql);
-  }  // end if no changed rows due to Name change during recording
+  dbQueue.push(std::move(sql));
 
   if (storage && storage->Id()) {
     sql = stringtf("UPDATE Storage SET DiskSpace = DiskSpace + %" PRIu64 " WHERE Id=%u", video_size, storage->Id());
-    zmDbDoUpdate(sql);
+    dbQueue.push(std::move(sql));
   }
 }  // Event::~Event()
 
@@ -392,7 +389,7 @@ void Event::AddPacket_(const std::shared_ptr<ZMPacket>packet) {
       Tag *tag = nullptr;
       auto tag_it = tags.find(cls);
       if (tag_it == tags.end()) {
-        Debug(1, "Tag not foudn %s", cls.c_str());
+        Debug(1, "Tag not found %s", cls.c_str());
         tag = Tag::find(cls);
         if (!tag) {
           tag = new Tag();
@@ -400,10 +397,13 @@ void Event::AddPacket_(const std::shared_ptr<ZMPacket>packet) {
           tag->save();
           Debug(1, "Created new Tag %s", cls.c_str());
           tags.emplace(std::make_pair(cls, *tag));
+          int tag_id = tag->Id();
+          delete tag;  // Delete after copying into map
+          tag = nullptr;
 
-          if (tag->Id()) {
-            // Store 
-            Event_Tag event_tag(tag->Id(), id, packet->timestamp);
+          if (tag_id) {
+            // Store
+            Event_Tag event_tag(tag_id, id, packet->timestamp);
             event_tag.save();
           }
         } else {
@@ -779,19 +779,17 @@ void Event::Run() {
       if (!packet->decoded) {
         Debug(1, "Not decoded");
         packet_lock.unlock();
-        // Stay behind decoder
-        Microseconds sleep_for = Microseconds(ZM_SAMPLE_RATE);
-        Debug(4, "Sleeping for %" PRId64 "us", int64(sleep_for.count()));
-        std::this_thread::sleep_for(sleep_for);
+        // Wait on packetqueue condition instead of blind sleep — wakes immediately
+        // when decoder sets decoded=true and calls packetqueue.notify_all()
+        packetqueue->wait_for(Microseconds(ZM_SAMPLE_RATE));
         continue;
       }
       if (!packet->analyzed) {
         Debug(1, "Not analyzed");
         packet_lock.unlock();
-        // Stay behind ai
-        Microseconds sleep_for = Microseconds(ZM_SAMPLE_RATE);
-        Debug(1, "Sleeping for %" PRId64 "us", int64(sleep_for.count()));
-        std::this_thread::sleep_for(sleep_for);
+        // Wait on packetqueue condition instead of blind sleep — wakes immediately
+        // when analysis sets analyzed=true and calls packetqueue.notify_all()
+        packetqueue->wait_for(Microseconds(ZM_SAMPLE_RATE));
         continue;
       }
 
@@ -817,7 +815,9 @@ void Event::Run() {
       packetqueue->increment_it(packetqueue_it, true);
     } else {
       if (terminate_ or zm_terminate) return;
-      usleep(10000);
+      // Wait on packetqueue condition instead of blind sleep — wakes immediately
+      // when a packet is queued or a packet lock becomes available
+      packetqueue->wait_for(Microseconds(10000));
     } // end if packet_lock
   }  // end while
 }  // end Run()
