@@ -25,6 +25,9 @@
 #include "zm_packet.h"
 #include "zm_signal.h"
 
+#include <climits>
+#include <vector>
+
 PacketQueue::PacketQueue():
   video_stream_id(-1),
   max_video_packet_count(-1),
@@ -36,7 +39,9 @@ PacketQueue::PacketQueue():
   warned_count(0),
   has_out_of_order_packets_(false),
   max_keyframe_interval_(0),
-  frames_since_last_keyframe_(0)
+  frames_since_last_keyframe_(0),
+  clear_packets_pending_(false),
+  next_queue_index_(0)
 {
 }
 
@@ -44,16 +49,17 @@ PacketQueue::PacketQueue():
  * Assumes first stream added will be the video stream
  */
 int PacketQueue::addStream() {
+  std::lock_guard<std::mutex> lck(mutex);
   deleting = false;
   if (max_stream_id == -1) {
     video_stream_id = 0;
     max_stream_id = 0;
   } else {
-    max_stream_id ++;
+    max_stream_id++;
   }
 
   packet_counts.reset(new int[max_stream_id + 1]);
-  for (int i=0; i <= max_stream_id; ++i)
+  for (int i = 0; i <= max_stream_id; ++i)
     packet_counts[i] = 0;
   return max_stream_id;
 }
@@ -74,6 +80,12 @@ PacketQueue::~PacketQueue() {
  */
 
 bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
+  std::vector<std::shared_ptr<ZMPacket>> packets_to_destroy;
+
+  {
+  std::unique_lock<std::mutex> lck(mutex);
+  if (deleting or zm_terminate) return false;
+
   if (iterators.empty()) {
     Debug(4, "No iterators so no one needs us to queue packets.");
     return false;
@@ -82,9 +94,6 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
     Debug(4, "No video keyframe so no one needs us to queue packets.");
     return false;
   }
-  {
-    std::unique_lock<std::mutex> lck(mutex);
-    if (deleting or zm_terminate) return false;
 
     AVPacket *add_avpacket = add_packet->packet.get();  // because std::shared_ptr accesses are way more expensive
     if (!has_out_of_order_packets_ and (add_avpacket->dts != AV_NOPTS_VALUE)) {
@@ -110,7 +119,7 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
       if (!add_packet->keyframe) {
         frames_since_last_keyframe_ ++;
         if (frames_since_last_keyframe_ > max_keyframe_interval_) {
-          max_keyframe_interval_ = frames_since_last_keyframe_; 
+          max_keyframe_interval_ = frames_since_last_keyframe_;
           Debug(1, "Have new keyframe interval %d", max_keyframe_interval_);
         }
       } else {
@@ -121,6 +130,7 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
       Debug(1, "Not video stream %d", add_avpacket->stream_index);
     }
 
+    add_packet->queue_index = next_queue_index_++;
     pktQueue.push_back(add_packet);
 
     /* Any iterators that are pointing to the end will now point to the newly pushed packet */
@@ -162,35 +172,27 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
 
         if (zm_packet->packet->stream_index == video_stream_id and zm_packet->keyframe) {
           for ( it = pktQueue.begin(); *it != zm_packet; ) {
-            it = this->deletePacket(it);
+            it = this->deletePacket(it, packets_to_destroy);
           }
           break;
         }  // end if erasing a whole gop
-        // Don't need to lock the new keyframe, just everything before it
-        ZMPacketLock packet_lock(zm_packet);
-        if (!packet_lock.trylock()) {
-          // If we can't get a lock, then we can't delete a GOP.  I'm actually not sure about this.
-          // It means someone is playing with the packet but doesn't mean we can't remove it from the queue.
-          if (warned_count < 2) {
-            warned_count++;
-            Warning("Found locked packet %d when trying to free up video packets. Our packet %d", zm_packet->image_index, add_packet->image_index);
-          }
-          break;
-        }
         ++it;
       }  // end foreach
     } else if (warned_count > 0) {
       warned_count--;
     }  // end if not able catch up
-    // We signal on every packet because someday we may analyze sound
-    Debug(4, "packetqueue queuepacket, unlocked signalling");
-    condition.notify_all();
-  }  // end lock scope
+
+  } // end scope for unique_lock — mutex released here
+
+  // We signal on every packet because someday we may analyze sound
+  Debug(4, "packetqueue queuepacket, unlocked signalling");
+  condition.notify_all();
+  // packets_to_destroy goes out of scope here, destroying packets without holding the mutex
 
   return true;
 }  // end bool PacketQueue::queuePacket(ZMPacket* zm_packet)
 
-packetqueue_iterator PacketQueue::deletePacket(packetqueue_iterator it) {
+packetqueue_iterator PacketQueue::deletePacket(packetqueue_iterator it, std::vector<std::shared_ptr<ZMPacket>> &deferred) {
   auto zm_packet = *it;
   for (
       auto iterators_it = iterators.begin();
@@ -210,7 +212,7 @@ packetqueue_iterator PacketQueue::deletePacket(packetqueue_iterator it) {
   zm_packet->notify_all();
 
   packet_counts[zm_packet->packet->stream_index] -= 1;
-  Debug(1,
+  Debug(4,
       "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
       zm_packet->packet->stream_index,
       zm_packet->image_index,
@@ -218,39 +220,50 @@ packetqueue_iterator PacketQueue::deletePacket(packetqueue_iterator it) {
       packet_counts[video_stream_id],
       max_video_packet_count,
       pktQueue.size());
+  // Move the shared_ptr into deferred so the packet is destroyed after the mutex is released
+  deferred.push_back(std::move(zm_packet));
   return pktQueue.erase(it);
 }
 
-void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
-  std::lock_guard<std::mutex> lck(mutex);
+bool PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
+  // Packets removed from the queue are collected here and destroyed after the
+  // mutex is released, so that expensive ZMPacket destructors (freeing images,
+  // av packets) don't run while the queue is locked.
+  std::vector<std::shared_ptr<ZMPacket>> packets_to_destroy;
+
+  {
+  std::unique_lock<std::mutex> lck(mutex);
   // Only do queueCleaning if we are adding a video keyframe, so that we guarantee that there is one.
   // No good.  Have to satisfy two conditions:
   // 1. packetqueue starts with a video keyframe
   // 2. Have minimum # of video packets
-  // 3. No packets can be locked
-  // 4. No iterator can point to one of the packets
+  // 3. No iterator can point to one of the packets
   //
   // So start at the beginning, counting video packets until the next keyframe.
   // Then if deleting those packets doesn't break 1 and 2, then go ahead and delete them.
   //
   // One assumption that we can make is that there will be packets in the queue. Because we call it while holding a locked packet
-  if (deleting) return;
+  if (deleting) return false;
 
+  // When keep_keyframes, normally only attempt clearing on video keyframes.
+  // If a previous clear attempt failed (clear_packets_pending_), retry on
+  // any video packet regardless of keyframe status.
   if (keep_keyframes and ! (
         add_packet->packet->stream_index == video_stream_id
         and
-        add_packet->keyframe
+        (add_packet->keyframe or clear_packets_pending_)
         and
         (packet_counts[video_stream_id] > pre_event_video_packet_count)
         and
         *(pktQueue.begin()) != add_packet
       )
      ) {
-    Debug(3, "stream index %d ?= video_stream_id %d, keyframe %d, keep_keyframes %d,  counts %d > pre_event_count %d at begin %d",
-          add_packet->packet->stream_index, video_stream_id, add_packet->keyframe, keep_keyframes, packet_counts[video_stream_id], pre_event_video_packet_count,
+    Debug(3, "stream index %d ?= video_stream_id %d, keyframe %d, keep_keyframes %d, pending %d, counts %d > pre_event_count %d at begin %d",
+          add_packet->packet->stream_index, video_stream_id, add_packet->keyframe, keep_keyframes, clear_packets_pending_,
+          packet_counts[video_stream_id], pre_event_video_packet_count,
           ( *(pktQueue.begin()) != add_packet )
          );
-    return;
+    return false;
   }
 
   // If analysis_it isn't at the end, we need to keep that many additional packets
@@ -262,19 +275,28 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   }
   Debug(1, "Tail count is %d, queue size is %zu, video_packets %d", tail_count, pktQueue.size(), packet_counts[video_stream_id]);
 
+  // Find the earliest queue_index that any iterator points to.  Since every
+  // packet now carries a monotonic queue_index assigned at enqueue time, this
+  // single integer comparison per packet replaces the previous per-packet
+  // is_there_an_iterator_pointing_to_packet() scan over all iterators.
+  uint64_t min_iterator_queue_index = UINT64_MAX;
+  for (auto iterators_it = iterators.begin(); iterators_it != iterators.end(); ++iterators_it) {
+    packetqueue_iterator *iterator_it = *iterators_it;
+    if (*iterator_it != pktQueue.end()) {
+      uint64_t qi = (*(*iterator_it))->queue_index;
+      if (qi < min_iterator_queue_index)
+        min_iterator_queue_index = qi;
+    }
+  }
+
   if (!keep_keyframes) {
     int packets_removed = 0;
     Debug(3, "Not keeping keyframes");
     // If not doing passthrough, we don't care about starting with a keyframe so logic is simpler
     while ((*pktQueue.begin() != add_packet) and (packet_counts[video_stream_id] > pre_event_video_packet_count + tail_count)) {
       std::shared_ptr<ZMPacket> zm_packet = *pktQueue.begin();
-      ZMPacketLock packet_lock(zm_packet);
-      if (!packet_lock.trylock()) {
-        Debug(1, "Failed locking packet %d", zm_packet->image_index);
-        break;
-      }
 
-      if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+      if (zm_packet->queue_index >= min_iterator_queue_index) {
         Debug(1, "Found iterator at beginning of queue.");
         break;
       }
@@ -283,7 +305,7 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
       pktQueue.pop_front();
       int stream_index = zm_packet->packet ? zm_packet->packet->stream_index : 0;
       packet_counts[stream_index] -= 1;
-      Debug(1,
+      Debug(4,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
             stream_index,
             zm_packet->image_index,
@@ -291,55 +313,42 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
             packet_counts[video_stream_id],
             pre_event_video_packet_count,
             pktQueue.size());
+      packets_to_destroy.push_back(std::move(zm_packet));
     } // end while
     Debug(3, "Done removing %d packets from queue. packet_counts %d >? pre_event %d + tail %d = %d",
         packets_removed, packet_counts[video_stream_id], pre_event_video_packet_count, tail_count, pre_event_video_packet_count + tail_count);
-    return;
+    clear_packets_pending_ = (packets_removed == 0);
+    // mutex released here at end of scope, packets destroyed after
+    return (packets_removed > 0);
   }
 
   auto it = pktQueue.begin();
   auto next_front = pktQueue.begin();
 
-  // First packet is special because we know it is a video keyframe and only need to check for lock
+  // First packet is special because we know it is a video keyframe and only need to check for iterator
   std::shared_ptr<ZMPacket> zm_packet = *it;
   if (zm_packet == add_packet) {
     Debug(1, "First packet in queue is the analysis packet.");
-    return;
+    return false;
   }
 
   int keyframe_interval_count = 0;
   int video_packets_to_delete = 0;    // This is a count of how many packets we will delete so we know when to stop looking
 
-  {
-    ZMPacketLock packet_lock(zm_packet);
-    if (!packet_lock.trylock()) {
-      Debug(4, "Failed getting lock on first packet");
-      return;
-    }  // end if first packet not locked
-   
-    if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
-      Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
-          video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
-      //zm_packet->unlock(); not necessary due to RAII
-      return;
-    }
-
-    ++it;
+  if (zm_packet->queue_index >= min_iterator_queue_index) {
+    Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
+        video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
+    clear_packets_pending_ = true;
+    return false;
   }
 
+  ++it;
+
   // Since we have many packets in the queue, we should NOT be pointing at end so don't need to test for that
   while (*it != add_packet) {
     zm_packet = *it;
-    {
-      // We are only locking it to see if anyone else is messing with it.
-      ZMPacketLock packet_lock(zm_packet);
-      if (!packet_lock.trylock()) {
-        Debug(3, "Failed locking packet %d", zm_packet->image_index);
-        break;
-      }
-    }
 
-    if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+    if (zm_packet->queue_index >= min_iterator_queue_index) {
       Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
           video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
       break;
@@ -377,7 +386,7 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
         continue;
       }
 
-      Debug(1,
+      Debug(4,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
             zm_packet->packet->stream_index,
             zm_packet->image_index,
@@ -387,18 +396,22 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
             pktQueue.size());
       pktQueue.pop_front();
       packet_counts[zm_packet->packet->stream_index] -= 1;
+      packets_to_destroy.push_back(std::move(zm_packet));
     }
-  }  // end if have at least max_video_packet_count video packets remaining
+    clear_packets_pending_ = false;
+  } else {
+    clear_packets_pending_ = true;
+  }
 
-  return;
-} // end voidPacketQueue::clearPackets(ZMPacket* zm_packet)
+  } // end scope for unique_lock — mutex released here
+  // packets_to_destroy goes out of scope here, destroying packets without holding the mutex
+  return !clear_packets_pending_;
+} // end bool PacketQueue::clearPackets(ZMPacket* zm_packet)
 
 void PacketQueue::stop() {
-  {
-    std::lock_guard<std::mutex> lck(mutex);
-    deleting = true;
-    condition.notify_all();
-  }
+  std::lock_guard<std::mutex> lck(mutex);
+  deleting = true;
+  condition.notify_all();
   for (const auto &p : pktQueue) {
     p->notify_all();
   }
@@ -439,8 +452,6 @@ void PacketQueue::clear() {
     *iterator_it = pktQueue.begin();
   }  // end foreach iterator
 
-  packet_counts.reset(new int[max_stream_id + 1]);
-
   packet_counts.reset();
   max_stream_id = -1;
   max_keyframe_interval_ = 0;
@@ -468,19 +479,13 @@ int PacketQueue::packet_count(int stream_id) {
 }  // end int PacketQueue::packet_count(int stream_id)
 
 ZMPacketLock PacketQueue::get_packet_no_wait(packetqueue_iterator *it) {
-
-  Debug(4, "Locking in get_packet using it %p queue end? %d",
-      std::addressof(*it), (*it == pktQueue.end()));
-
   std::unique_lock<std::mutex> lck(mutex);
-  Debug(4, "Have Lock in get_packet");
+  Debug(4, "get_packet_no_wait using it %p queue end? %d",
+      std::addressof(*it), (*it == pktQueue.end()));
   if (deleting or zm_terminate)
     return ZMPacketLock();
-  if ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
-    Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
-    condition.wait(lck, [&]{ return (*it != pktQueue.end()) || deleting || zm_terminate; });
-  }
-  if ((*it == pktQueue.end()) or deleting or zm_terminate) return ZMPacketLock();
+  if (*it == pktQueue.end())
+    return ZMPacketLock();
 
   std::shared_ptr<ZMPacket> p = *(*it);
   ZMPacketLock packet_lock(p);
@@ -497,13 +502,11 @@ ZMPacketLock PacketQueue::get_packet(packetqueue_iterator *it) {
   if (deleting or zm_terminate)
     return ZMPacketLock();
 
-  Debug(4, "Locking in get_packet using it %p queue end? %d",
-        std::addressof(*it), (*it == pktQueue.end()));
-
   {
     // scope for lock
     std::unique_lock<std::mutex> lck(mutex);
-    Debug(4, "Have Lock in get_packet");
+    Debug(4, "get_packet using it %p queue end? %d",
+          std::addressof(*it), (*it == pktQueue.end()));
     while (!(deleting or zm_terminate)) {
       while ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
         Debug(4, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
@@ -542,13 +545,11 @@ ZMPacketLock PacketQueue::get_packet_and_increment_it(packetqueue_iterator *it)
   if (deleting or zm_terminate)
     return ZMPacketLock();
 
-  Debug(4, "Locking in get_packet using it %p queue end? %d",
-        std::addressof(*it), (*it == pktQueue.end()));
-
   {
     // scope for lock
     std::unique_lock<std::mutex> lck(mutex);
-    Debug(4, "Have Lock in get_packet");
+    Debug(4, "get_packet_and_increment_it using it %p queue end? %d",
+          std::addressof(*it), (*it == pktQueue.end()));
     while (!(deleting or zm_terminate)) {
       while ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
         Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
@@ -584,12 +585,21 @@ void PacketQueue::unlock(ZMPacketLock *lp) {
   condition.notify_all();
 }
 
-bool PacketQueue::increment_it(packetqueue_iterator *it) {
-  std::lock_guard<std::mutex> lck(mutex);
+bool PacketQueue::increment_it(packetqueue_iterator *it, bool wait) {
+  std::unique_lock<std::mutex> lck(mutex);
   Debug(2, "Incrementing %p, queue size %zu, end? %d, deleting %d", it, pktQueue.size(), ((*it) == pktQueue.end()), deleting);
-  if (((*it) == pktQueue.end()) or deleting) {
-    if (!deleting) Warning("increment_it at end!");
-    return false;
+  if (wait) {
+    while ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
+      Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
+      condition.wait(lck);
+      Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
+    }
+    if (deleting or zm_terminate) return false;
+  } else {
+    if (((*it) == pktQueue.end()) or deleting) {
+      if (!deleting) Debug(1, "increment_it at end!");
+      return false;
+    }
   }
   ++(*it);
   if (*it != pktQueue.end()) {
@@ -602,9 +612,9 @@ bool PacketQueue::increment_it(packetqueue_iterator *it) {
 
 // Increment it only considering packets for a given stream
 bool PacketQueue::increment_it(packetqueue_iterator *it, int stream_id) {
+  std::lock_guard<std::mutex> lck(mutex);
   Debug(2, "Incrementing %p, queue size %zu, end? %d", it, pktQueue.size(), (*it == pktQueue.end()));
 
-  std::lock_guard<std::mutex> lck(mutex);
   if (*it == pktQueue.end()) {
     return false;
   }
@@ -655,6 +665,11 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
   }
 
   // it either points to beginning or we have seen pre_event_count video packets.
+  // The loop above doesn't count the begin packet, so check it now.
+  if (pre_event_count and (packet->packet->stream_index == video_stream_id)) {
+    pre_event_count--;
+  }
+
   if (pre_event_count) {
     if (packet->image_index < (int)pre_event_count) {
       // probably just starting up
@@ -684,8 +699,8 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
 }  // end packetqueue_iterator *PacketQueue::get_event_start_packet_it
 
 void PacketQueue::dumpQueue() {
-  std::list<std::shared_ptr<ZMPacket>>::reverse_iterator it;
-  for ( it = pktQueue.rbegin(); it != pktQueue.rend(); ++ it ) {
+  std::lock_guard<std::mutex> lck(mutex);
+  for (auto it = pktQueue.rbegin(); it != pktQueue.rend(); ++it) {
     std::shared_ptr<ZMPacket> zm_packet = *it;
     ZM_DUMP_PACKET(zm_packet->packet, is_there_an_iterator_pointing_to_packet(zm_packet) ? "*" : "");
   }
@@ -695,10 +710,10 @@ void PacketQueue::dumpQueue() {
  * nullptr if no keyframe video packet exists.
  */
 packetqueue_iterator * PacketQueue::get_video_it(bool wait) {
+  std::unique_lock<std::mutex> lck(mutex);
+
   packetqueue_iterator *it = new packetqueue_iterator;
   iterators.push_back(it);
-
-  std::unique_lock<std::mutex> lck(mutex);
   *it = pktQueue.begin();
 
   if (wait) {
@@ -708,6 +723,8 @@ packetqueue_iterator * PacketQueue::get_video_it(bool wait) {
       *it = pktQueue.begin();
     }
     if (deleting or zm_terminate) {
+      iterators.remove(it);
+      delete it;
       return nullptr;
     }
   }
@@ -716,6 +733,8 @@ packetqueue_iterator * PacketQueue::get_video_it(bool wait) {
     std::shared_ptr<ZMPacket> zm_packet = *(*it);
     if (!zm_packet) {
       Error("Null zmpacket in queue!?");
+      iterators.remove(it);
+      delete it;
       return nullptr;
     }
     Debug(1, "Packet keyframe %d for stream %d, so returning the it to it",
@@ -766,12 +785,15 @@ bool PacketQueue::is_there_an_iterator_pointing_to_packet(const std::shared_ptr<
 }
 
 void PacketQueue::setMaxVideoPackets(int p) {
+  std::lock_guard<std::mutex> lck(mutex);
   max_video_packet_count = p;
   Debug(1, "Setting max_video_packet_count to %d", p);
   if (max_video_packet_count < 0)
-    max_video_packet_count = 0 ;
+    max_video_packet_count = 0;
 }
+
 void PacketQueue::setPreEventVideoPackets(int p) {
+  std::lock_guard<std::mutex> lck(mutex);
   pre_event_video_packet_count = p;
   Debug(1, "Setting pre_event_video_packet_count to %d", p);
   if (pre_event_video_packet_count < 1)
@@ -787,3 +809,8 @@ void PacketQueue::wait() {
   std::unique_lock<std::mutex> lck(mutex);
   condition.wait(lck);
 }
+
+void PacketQueue::wait_for(Microseconds duration) {
+  std::unique_lock<std::mutex> lck(mutex);
+  condition.wait_for(lck, duration);
+}
