@@ -48,6 +48,17 @@ std::string remove_newlines(std::string input);
 std::string Trim(const std::string &str, const std::string &char_set);
 inline std::string TrimSpaces(const std::string &str) { return Trim(str, " \t"); }
 std::string ReplaceAll(std::string str, const std::string &old_value, const std::string &new_value);
+inline std::string ShellEscape(const std::string &str) {
+  std::string result = "'";
+  for (char c : str) {
+    if (c == '\'')
+      result += "'\\''";
+    else
+      result += c;
+  }
+  result += "'";
+  return result;
+}
 inline std::string StringToUpper(std::string str) {
   std::transform(str.begin(), str.end(), str.begin(), ::toupper);
   return str;
