@@ -19,9 +19,9 @@
 //
 
 
-if ( $action == 'logout' ) {
+if ($action == 'logout') {
   userLogout();
-  $view = 'login';
+  $redirect = '?vew=login';
 } elseif ( $action == 'config' ) {
   $redirect = '?view=user&prev=console&uid='.$user->Id();
 }
