@@ -22,15 +22,94 @@ require_once('session.php');
 require_once('User.php');
 require_once('Group_Permission.php');
 require_once('Monitor_Permission.php');
+
+require_once('User_Role.php');
+require_once('Role_Group_Permission.php');
+require_once('Role_Monitor_Permission.php');
 require_once(__DIR__.'/../vendor/autoload.php');
-use \Firebase\JWT\JWT;
+
+if (ZM_AUTH_TYPE == 'oAuth') {
+  require_once 'oAuth.php';
+
+  $oAuth = new oAuth();
+  $oAuthSession = $oAuth->oAuthSession;
+  if ($oAuthSession) {
+    if (isset($_GET['code'])) {
+      if (!$oAuth->check_code($_GET['code'])) {
+        ZM\Debug('Redir to where we came from'.$oAuthSession['Redir']);
+        #$oAuth->logout();
+        global $redirect;
+        $redirect = $oAuthSession['Redir'];
+        return;
+      }
+    } else if (isset($_POST['action']) and ($_POST['action'] == 'login')) {
+      $oAuth->login($_POST['username'], $_POST['password']);
+    } else if (isset($_POST['action']) and ($_POST['action'] == 'logout')) {
+      $oAuth->logout();
+      userLogout();
+      return;
+    }
+
+    if ($oAuth->isLoggedIn) {
+      $username = $oAuth->preferred_username();
+      if ($username) {
+        global $user;
+        $user = ZM\User::find_one(['Username'=>$username]);
+
+        if (!$user) {
+          $user = ZM\User::find_one(['Username'=>'default']);
+          if (!$user) $user = new ZM\User();
+          $user->Id(null);
+          $user->save([
+            'Username'  => $username,
+            'Name'      => $profile->displayName,
+            'Email'     => to_string($profile->mail),
+            'Phone'     => to_string($profile->businessPhones),
+          ]);
+        } else {
+          $profile = $oAuth->Graph()->getProfile();
+          # Check for new values
+          $changes = $user->changes([
+            'Name'      => $profile->displayName,
+            'Email'     => to_string($profile->mail),
+            'Phone'     => to_string($profile->businessPhones),
+          ]);
+          if ($changes != []) {
+            $user->save($changes);
+          }
+        }
+          #$groups = $oAuth->Graph()->getMyGroups();
+          #foreach ($groups as $group) {
+            #ZM\Debug(print_r($group, true));
+          #}
+        $_SESSION['username'] = $user->Username();
+        generateAuthHash(ZM_AUTH_HASH_IPS);
+
+        if (isset($_GET['code'])) {
+          global $redirect;
+          $redirect = '?view='.$user->HomeView();
+        }
+      } # end if username
+    } else {
+      $oAuth->redirect_to_login();
+    }
+  } else {
+    ZM\Error("No oAuthSession");
+    header('Location: /');
+    session_write_close();
+    exit;
+  }
+}
 
 function password_type($password) {
-   if ( $password[0] == '*' ) {
+  if (!$password || $password === '') {
+    return 'plain';
+  }
+  if ($password[0] == '*') {
     return 'mysql';
-  } else if ( preg_match('/^\$2[ayb]\$.+$/', $password) ) {
+  } else if (preg_match('/^\$2[ayb]\$.+$/', $password)) {
     return 'bcrypt';
-  } else if ( substr($password, 0,4) == '-ZM-' ) {
+  } else if (substr($password, 0, 4) == '-ZM-') {
   // zmupdate.pl adds a '-ZM-' prefix to overlay encrypted passwords
   // this is done so that we don't spend cycles doing two bcrypt password_verify calls
   // for every wrong password entered. This will only be invoked for passwords zmupdate.pl has
@@ -43,18 +122,16 @@ function password_type($password) {
 // this function migrates mysql hashing to bcrypt, if you are using PHP >= 5.5
 // will be called after successful login, only if mysql hashing is detected
 function migrateHash($user, $pass) {
-  if ( function_exists('password_hash') ) {
+  if (function_exists('password_hash')) {
     ZM\Info("Migrating $user to bcrypt scheme");
     // let it generate its own salt, and ensure bcrypt as PASSWORD_DEFAULT may change later
-    // we can modify this later to support argon2 etc as switch to its own password signature detection 
+    // we can modify this later to support argon2 etc as switch to its own password signature detection
     $bcrypt_hash = password_hash($pass, PASSWORD_BCRYPT);
-    //ZM\Info ("hased bcrypt $pass is $bcrypt_hash");
-    $update_password_sql = 'UPDATE Users SET Password=\''.$bcrypt_hash.'\' WHERE Username=\''.$user.'\'';
-    dbQuery($update_password_sql);
+    dbQuery('UPDATE Users SET Password=? WHERE Username=?', array($bcrypt_hash, $user));
     # Since password field has changed, existing auth_hash is no longer valid
     generateAuthHash(ZM_AUTH_HASH_IPS, true);
   } else {
-    ZM\Info('Cannot migrate password scheme to bcrypt, as you are using PHP < 5.3');
+    ZM\Info('Cannot migrate password scheme to bcrypt, as you are using PHP < 5.5');
     return;
   }
 }
@@ -69,7 +146,7 @@ function validateUser($username='', $password='') {
   // local user, shouldn't affect the global user
   $user = dbFetchOne($sql, NULL, array($username)); // Not global
   if (!$user) {
-    return array(false, "Could not retrieve user $username details");
+    return array(false, 'Invalid username or password');
   }
 
   switch (password_type($user['Password'])) {
@@ -105,7 +182,7 @@ function validateUser($username='', $password='') {
   if ($password_correct) {
     return array(new ZM\User($user), 'OK');
   }
-  return array(false, "Login denied for user \"$username\"");
+  return array(false, 'Invalid username or password');
 } # end function validateUser
 
 function userLogout() {
@@ -120,7 +197,8 @@ function validateToken($token, $allowed_token_type='access') {
   $key = ZM_AUTH_HASH_SECRET;
   //if (ZM_AUTH_HASH_IPS) $key .= $_SERVER['REMOTE_ADDR'];
   try {
-    $decoded_token = JWT::decode($token, $key, array('HS256'));
+    $headers = new \stdClass();
+    $decoded_token = Firebase\JWT\JWT::decode($token, new Firebase\JWT\Key($key, 'HS256'), $headers);
   } catch (Exception $e) {
     ZM\Error("Unable to authenticate user. error decoding JWT token:".$e->getMessage());
     return array(false, $e->getMessage());
@@ -223,6 +301,9 @@ function getAuthUser($auth) {
   return null;
 } // end getAuthUser($auth)
 
+// TODO: MD5 is cryptographically weak. Consider migrating to hash('sha256', ...)
+// However, changing this would invalidate all existing auth hashes and require
+// a coordinated update of all components that generate/validate auth hashes.
 function calculateAuthHash($remoteAddr='') {
   global $user;
   $local_time = localtime();
@@ -235,32 +316,28 @@ function generateAuthHash($useRemoteAddr, $force=false) {
   global $user;
   if (!isset($_SESSION['remoteAddr'])) $_SESSION['remoteAddr'] = '';
   if (ZM_OPT_USE_AUTH and (ZM_AUTH_RELAY == 'hashed') and $user and $user->Username()) {
-    if (!isset($_SESSION)) {
-      # Appending the remoteAddr prevents us from using an auth hash generated for a different ip
-      #$auth = calculateAuthHash($useRemoteAddr?$_SESSION['remoteAddr']:'');
-      $auth = calculateAuthHash();
-      return $auth;
-    } else {
-      $time = time();
-      # We use 1800 so that we regenerate the hash at half the TTL
-      $mintime = $time - (ZM_AUTH_HASH_TTL * 1800);
-      $remoteAddr = ZM_AUTH_HASH_IPS ? $_SESSION['remoteAddr'] : '';
-      # Appending the remoteAddr prevents us from using an auth hash generated for a different ip
-      if ($force or ( !isset($_SESSION['AuthHash'.$remoteAddr]) ) or ( $_SESSION['AuthHashGeneratedAt'] < $mintime )) {
-        $auth = calculateAuthHash($useRemoteAddr ? $remoteAddr : '');
-        # Don't both regenerating Auth Hash if an hour hasn't gone by yet
-        $_SESSION['AuthHash'.$remoteAddr] = $auth;
-        $_SESSION['AuthHashGeneratedAt'] = $time;
-        # Because we don't write out the session, it shouldn't actually get written out to disk.  However if it does, the GeneratedAt should protect us.
-      } # end if AuthHash is not cached
-      return $_SESSION['AuthHash'.$remoteAddr];
-    }
+    $time = time();
+    # We use 1800 so that we regenerate the hash at half the TTL
+    $mintime = $time - (ZM_AUTH_HASH_TTL * 1800);
+    $remoteAddr = ZM_AUTH_HASH_IPS ? $_SESSION['remoteAddr'] : '';
+    # Appending the remoteAddr prevents us from using an auth hash generated for a different ip
+    if ($force or (!isset($_SESSION['AuthHash'.$remoteAddr])) or ($_SESSION['AuthHashGeneratedAt'] < $mintime)) {
+      $auth = calculateAuthHash($useRemoteAddr ? $remoteAddr : '');
+      # Don't both regenerating Auth Hash if an hour hasn't gone by yet
+      $_SESSION['AuthHash'.$remoteAddr] = $auth;
+      $_SESSION['AuthHashGeneratedAt'] = $time;
+      # Because we don't write out the session, it shouldn't actually get written out to disk.  However if it does, the GeneratedAt should protect us.
+    } # end if AuthHash is not cached
+    return $_SESSION['AuthHash'.$remoteAddr];
   } # end if using AUTH and AUTH_RELAY
   return '';
 }
 
 $group_permissions = null;
 $monitor_permissions = null; # hash indexed by MonitorId
+$role_group_permissions = null;
+$role_monitor_permissions = null;
+
 function visibleMonitor($mid) {
   global $user;
   if (!$user) return false;
@@ -275,7 +352,7 @@ function visibleMonitor($mid) {
   if (isset($monitor_permissions[$mid])) {
     $permission = $monitor_permissions[$mid]->Permission();
     if ($permission != 'Inherit') {
-      ZM\Debug('Returning '.($permission == 'None' ? false : true)." for monitor $mid");
+      ZM\Debug('Returning '.($permission == 'None' ? false : true)." for monitor $mid from user monitor permission");
       return ($permission == 'None' ? false : true);
     }
   }
@@ -284,12 +361,12 @@ function visibleMonitor($mid) {
   if ($group_permissions === null)
     $group_permissions = $user->Group_Permissions();
 
-  # If denied view in any group, then can't view it.
+  # Check user's group permissions
   $group_permission_value = 'Inherit';
   foreach ($group_permissions as $permission) {
     $value = $permission->MonitorPermission($mid);
     if ($value == 'None') {
-      ZM\Debug("Can't view monitor $mid because of group ".$permission->Group()->Name().' '.$permission->Permission());
+      ZM\Debug("Can't view monitor $mid because of user group ".$permission->Group()->Name().' '.$permission->Permission());
       return false;
     } else if ($value == 'View' or $value == 'Edit') {
       $group_permission_value = $value;
@@ -297,6 +374,46 @@ function visibleMonitor($mid) {
   }
   if ($group_permission_value != 'Inherit') return true;
 
+  # Check role permissions if user has a role
+  $role = $user->Role();
+  if ($role) {
+    global $role_monitor_permissions;
+    if ($role_monitor_permissions === null) {
+      $role_monitor_permissions = array_to_hash_by_key('MonitorId', $role->Monitor_Permissions());
+    }
+
+    if (isset($role_monitor_permissions[$mid])) {
+      $permission = $role_monitor_permissions[$mid]->Permission();
+      if ($permission != 'Inherit') {
+        ZM\Debug('Returning '.($permission == 'None' ? false : true)." for monitor $mid from role monitor permission");
+        return ($permission == 'None' ? false : true);
+      }
+    }
+
+    global $role_group_permissions;
+    if ($role_group_permissions === null)
+      $role_group_permissions = $role->Group_Permissions();
+
+    # Check role's group permissions
+    $role_group_permission_value = 'Inherit';
+    foreach ($role_group_permissions as $permission) {
+      $value = $permission->MonitorPermission($mid);
+      if ($value == 'None') {
+        ZM\Debug("Can't view monitor $mid because of role group ".$permission->Group()->Name().' '.$permission->Permission());
+        return false;
+      } else if ($value == 'View' or $value == 'Edit') {
+        $role_group_permission_value = $value;
+      }
+    }
+    if ($role_group_permission_value != 'Inherit') return true;
+
+    # Check role's base Monitors permission if user's is 'None'
+    if ($user->Monitors() == 'None' and $role->Monitors() != 'None') {
+      ZM\Debug("Using role Monitors permission ".$role->Monitors()." for monitor $mid");
+      return true;
+    }
+  }
+
   return ($user->Monitors() != 'None');
 }
 
@@ -304,7 +421,19 @@ function canView($area, $mid=false) {
   global $user;
   if (!$user) return false;
   if ($mid) return visibleMonitor($mid);
-  return ($user->$area() && ($user->$area() != 'None'));
+
+  # Check user's direct permission first
+  if ($user->$area() && ($user->$area() != 'None')) {
+    return true;
+  }
+
+  # If user's permission is 'None', check role permission
+  $role = $user->Role();
+  if ($role && $role->$area() && ($role->$area() != 'None')) {
+    return true;
+  }
+
+  return false;
 }
 
 function editableMonitor($mid) {
@@ -320,27 +449,66 @@ function editableMonitor($mid) {
   if ($monitor_permissions === null) {
     $monitor_permissions = array_to_hash_by_key('MonitorId', ZM\Monitor_Permission::find(array('UserId'=>$user->Id())));
   }
-  if (isset($monitor_permissions[$mid]) and 
-    ($monitor_permissions[$mid]->Permission() == 'None' or $monitor_permissions[$mid]->Permission() == 'View')
-  ) {
-    //ZM\Debug("Have monitor permission == ".$monitor_permissions[$mid]->Permission());
-    return false;
+  if (isset($monitor_permissions[$mid])) {
+    $permission = $monitor_permissions[$mid]->Permission();
+    if ($permission == 'None' or $permission == 'View') {
+      return false;
+    } else if ($permission == 'Edit') {
+      return true;
+    }
   }
 
   global $group_permissions;
   if ($group_permissions === null)
     $group_permissions = ZM\Group_Permission::find(array('UserId'=>$user->Id()));
 
-  # If denied view in any group, then can't view it.
+  # Check user's group permissions
   foreach ($group_permissions as $permission) {
     $perm_value = $permission->MonitorPermission($mid);
-    //ZM\Debug("Have group permission $perm_value");
     if ($perm_value == 'Edit') {
       return true;
     }
   }
 
-  #ZM\Debug("Monitors permission is ".$user->Monitors());
+  # Check role permissions if user has a role
+  $role = $user->Role();
+  if ($role) {
+    global $role_monitor_permissions;
+    if ($role_monitor_permissions === null) {
+      $role_monitor_permissions = array_to_hash_by_key('MonitorId', $role->Monitor_Permissions());
+    }
+
+    if (isset($role_monitor_permissions[$mid])) {
+      $permission = $role_monitor_permissions[$mid]->Permission();
+      if ($permission == 'None' or $permission == 'View') {
+        return false;
+      } else if ($permission == 'Edit') {
+        return true;
+      }
+    }
+
+    global $role_group_permissions;
+    if ($role_group_permissions === null)
+      $role_group_permissions = $role->Group_Permissions();
+
+    # Check role's group permissions
+    foreach ($role_group_permissions as $permission) {
+      $perm_value = $permission->MonitorPermission($mid);
+      if ($perm_value == 'Edit') {
+        return true;
+      }
+    }
+
+    # Check role's base Monitors permission if user's doesn't allow edit
+    $user_monitors = $user->Monitors();
+    if ($user_monitors != 'Edit' && $user_monitors != 'Create') {
+      $role_monitors = $role->Monitors();
+      if ($role_monitors == 'Edit' || $role_monitors == 'Create') {
+        return true;
+      }
+    }
+  }
+
   return (($user->Monitors() == 'Edit') || ($user->Monitors() == 'Create'));
 }
 
@@ -349,13 +517,37 @@ function canEdit($area, $mid=false) {
 
   if (!$user) return false;
   if ($mid) return editableMonitor($mid);
-  return ($user->$area() == 'Edit' or $user->$area() == 'Create');
+
+  # Check user's direct permission first
+  if ($user->$area() == 'Edit' or $user->$area() == 'Create') {
+    return true;
+  }
+
+  # If user's permission doesn't allow edit, check role permission
+  $role = $user->Role();
+  if ($role && ($role->$area() == 'Edit' or $role->$area() == 'Create')) {
+    return true;
+  }
+
+  return false;
 }
 
 function canCreate($area) {
   global $user;
+  if (!$user) return false;
 
-  return ( $user && ($user->$area() == 'Create') );
+  # Check user's direct permission first
+  if ($user->$area() == 'Create') {
+    return true;
+  }
+
+  # If user's permission doesn't allow create, check role permission
+  $role = $user->Role();
+  if ($role && $role->$area() == 'Create') {
+    return true;
+  }
+
+  return false;
 }
 
 function userFromSession() {
@@ -411,108 +603,109 @@ if (ZM_OPT_USE_AUTH) {
     $user = $ret[0];
   } else {
     // Non token based auth
-
-    if (ZM_AUTH_HASH_LOGINS && empty($user) && !empty($_REQUEST['auth'])) {
-      $user = getAuthUser($_REQUEST['auth']);
-      if ($user) {
-        $remoteAddr = ZM_AUTH_HASH_IPS ? $_SESSION['remoteAddr'] : '';
-        if (isset($_SESSION['AuthHash'.$remoteAddr]) and ($_SESSION['AuthHash'.$remoteAddr] != $_REQUEST['auth'])) {
-          unset($_SESSION['AuthHashGeneratedAt']);
-          unset($_SESSION['AuthHash'.$remoteAddr]);
+    if (empty($user)) {
+      if (ZM_AUTH_HASH_LOGINS && !empty($_REQUEST['auth'])) {
+        $user = getAuthUser($_REQUEST['auth']);
+        if ($user) {
+          $remoteAddr = ZM_AUTH_HASH_IPS ? $_SESSION['remoteAddr'] : '';
+          if (isset($_SESSION['AuthHash'.$remoteAddr]) and ($_SESSION['AuthHash'.$remoteAddr] != $_REQUEST['auth'])) {
+            unset($_SESSION['AuthHashGeneratedAt']);
+            unset($_SESSION['AuthHash'.$remoteAddr]);
+          }
+          $_SESSION['username'] = $user->Username();
         }
-        $_SESSION['username'] = $user->Username();
-      }
-    } else if (!(empty($_REQUEST['user']) or empty($_REQUEST['pass']))) {
-      # The shortened versions are used in auth_relay = PLAIN
-      $ret = validateUser($_REQUEST['user'], $_REQUEST['pass']);
-      if (!$ret[0]) {
-        ZM\Warning($ret[1]);
-        unset($user); // unset should be ok here because we aren't in a function
-        return;
-      }
-      $user = $ret[0];
-    } else if (!(empty($_REQUEST['username']) or empty($_REQUEST['password']))) {
-      # Longer versions are used on login page
-      $ret = validateUser($_REQUEST['username'], $_REQUEST['password']);
-      if (!$ret[0]) {
-        ZM\Warning($ret[1]);
-        unset($user); // unset should be ok here because we aren't in a function
-        return;
-      }
-      $user = $ret[0];
-
-      if (
-        defined('ZM_OPT_USE_GOOG_RECAPTCHA') && ZM_OPT_USE_GOOG_RECAPTCHA
-        && defined('ZM_OPT_GOOG_RECAPTCHA_SECRETKEY') && ZM_OPT_GOOG_RECAPTCHA_SECRETKEY
-        && defined('ZM_OPT_GOOG_RECAPTCHA_SITEKEY') && ZM_OPT_GOOG_RECAPTCHA_SITEKEY
-      ) {
-        if ( !isset($_REQUEST['g-recaptcha-response']) ) {
-          ZM\Error('reCaptcha authentication failed. No g-recpatcha-response in REQUEST: ');
+      } else if (!(empty($_REQUEST['user']) or empty($_REQUEST['pass']))) {
+        # The shortened versions are used in auth_relay = PLAIN
+        $ret = validateUser($_REQUEST['user'], $_REQUEST['pass']);
+        if (!$ret[0]) {
+          ZM\Warning($ret[1]);
           unset($user); // unset should be ok here because we aren't in a function
           return;
         }
-        $url = 'https://www.google.com/recaptcha/api/siteverify';
-        $fields = array (
-          'secret'    => ZM_OPT_GOOG_RECAPTCHA_SECRETKEY,
-          'response'  => $_REQUEST['g-recaptcha-response'],
-          'remoteip'  => $_SERVER['REMOTE_ADDR']
-        );
-        $res = do_post_request($url, http_build_query($fields));
-        $responseData = json_decode($res, true);
-        // credit: https://github.com/google/recaptcha/blob/master/src/ReCaptcha/Response.php
-        // if recaptcha resulted in error, we might have to deny login
-        if ( isset($responseData['success']) && ($responseData['success'] == false) ) {
-          // PP - before we deny auth, let's make sure the error was not 'invalid secret'
-          // because that means the user did not configure the secret key correctly
-          // in this case, we prefer to let him login in and display a message to correct
-          // the key. Unfortunately, there is no way to check for invalid site key in code
-          // as it produces the same error as when you don't answer a recaptcha
-          if ( isset($responseData['error-codes']) && is_array($responseData['error-codes']) ) {
-            if ( !in_array('invalid-input-secret', $responseData['error-codes']) ) {
-              ZM\Error('reCaptcha authentication failed. response was: ' . print_r($responseData['error-codes'],true));
-              unset($user); // unset should be ok here because we aren't in a function
-              return;
-            } else {
-              ZM\Error('Invalid recaptcha secret detected');
-            }
+        $user = $ret[0];
+      } else if (!(empty($_REQUEST['username']) or empty($_REQUEST['password']))) {
+        # Longer versions are used on login page
+        $ret = validateUser($_REQUEST['username'], $_REQUEST['password']);
+        if (!$ret[0]) {
+          ZM\Warning($ret[1]);
+          unset($user); // unset should be ok here because we aren't in a function
+          return;
+        }
+        $user = $ret[0];
+
+        if (
+          defined('ZM_OPT_USE_GOOG_RECAPTCHA') && ZM_OPT_USE_GOOG_RECAPTCHA
+          && defined('ZM_OPT_GOOG_RECAPTCHA_SECRETKEY') && ZM_OPT_GOOG_RECAPTCHA_SECRETKEY
+          && defined('ZM_OPT_GOOG_RECAPTCHA_SITEKEY') && ZM_OPT_GOOG_RECAPTCHA_SITEKEY
+        ) {
+          if ( !isset($_REQUEST['g-recaptcha-response']) ) {
+            ZM\Error('reCaptcha authentication failed. No g-recpatcha-response in REQUEST: ');
+            unset($user); // unset should be ok here because we aren't in a function
+            return;
           }
-        } // end if success==false
-      } // end if using reCaptcha
+          $url = 'https://www.google.com/recaptcha/api/siteverify';
+          $fields = array (
+            'secret'    => ZM_OPT_GOOG_RECAPTCHA_SECRETKEY,
+            'response'  => $_REQUEST['g-recaptcha-response'],
+            'remoteip'  => $_SERVER['REMOTE_ADDR']
+          );
+          $res = do_post_request($url, http_build_query($fields));
+          $responseData = json_decode($res, true);
+          // credit: https://github.com/google/recaptcha/blob/master/src/ReCaptcha/Response.php
+          // if recaptcha resulted in error, we might have to deny login
+          if ( isset($responseData['success']) && ($responseData['success'] == false) ) {
+            // PP - before we deny auth, let's make sure the error was not 'invalid secret'
+            // because that means the user did not configure the secret key correctly
+            // in this case, we prefer to let him login in and display a message to correct
+            // the key. Unfortunately, there is no way to check for invalid site key in code
+            // as it produces the same error as when you don't answer a recaptcha
+            if ( isset($responseData['error-codes']) && is_array($responseData['error-codes']) ) {
+              if ( !in_array('invalid-input-secret', $responseData['error-codes']) ) {
+                ZM\Error('reCaptcha authentication failed. response was: ' . print_r($responseData['error-codes'],true));
+                unset($user); // unset should be ok here because we aren't in a function
+                return;
+              } else {
+                ZM\Error('Invalid recaptcha secret detected');
+              }
+            }
+          } // end if success==false
+        } // end if using reCaptcha
 
-      zm_session_clear(); # Closes session
-      zm_session_regenerate_id(); # starts session
+        zm_session_clear(); # Closes session
+        zm_session_regenerate_id(); # starts session
 
-      $username = $_REQUEST['username'];
-      $password = $_REQUEST['password'];
+        $username = $_REQUEST['username'];
+        $password = $_REQUEST['password'];
 
-      ZM\Info("Login successful for user \"$username\"");
-      $password_type = password_type($user->Password());
+        ZM\Info("Login successful for user \"$username\"");
+        $password_type = password_type($user->Password());
 
-      if ( $password_type == 'mysql' or $password_type == 'mysql+bcrypt' ) {
-        ZM\Info('Migrating password, if possible for future logins');
-        migrateHash($username, $password);
-      }
+        if ( $password_type == 'mysql' or $password_type == 'mysql+bcrypt' ) {
+          ZM\Info('Migrating password, if possible for future logins');
+          migrateHash($username, $password);
+        }
 
-      if (ZM_AUTH_TYPE == 'builtin') {
-        $_SESSION['passwordHash'] = $user->Password();
-      }
+        if (ZM_AUTH_TYPE == 'builtin') {
+          $_SESSION['passwordHash'] = $user->Password();
+        }
 
-      $_SESSION['username'] = $user->Username();
-      if (ZM_AUTH_RELAY == 'plain') {
-        // Need to save this in session, can't use the value in User because it is hashed
-        $_SESSION['password'] = $_REQUEST['password'];
-      }
-    } else if ((ZM_AUTH_TYPE == 'remote') and !empty($_SERVER['REMOTE_USER'])) {
-      if (ZM_CASE_INSENSITIVE_USERNAMES) {
-        $sql = 'SELECT * FROM Users WHERE Enabled=1 AND LOWER(Username)=LOWER(?)';
+        $_SESSION['username'] = $user->Username();
+        if (ZM_AUTH_RELAY == 'plain') {
+          // Need to save this in session, can't use the value in User because it is hashed
+          $_SESSION['password'] = $_REQUEST['password'];
+        }
+      } else if ((ZM_AUTH_TYPE == 'remote') and !empty($_SERVER['REMOTE_USER'])) {
+        if (ZM_CASE_INSENSITIVE_USERNAMES) {
+          $sql = 'SELECT * FROM Users WHERE Enabled=1 AND LOWER(Username)=LOWER(?)';
+        } else {
+          $sql = 'SELECT * FROM Users WHERE Enabled=1 AND Username=?';
+        }
+        // local user, shouldn't affect the global user
+        $user = new ZM\User(dbFetchOne($sql, NULL, array($_SERVER['REMOTE_USER'])));
       } else {
-        $sql = 'SELECT * FROM Users WHERE Enabled=1 AND Username=?';
+        $user = userFromSession();
       }
-      // local user, shouldn't affect the global user
-      $user = new ZM\User(dbFetchOne($sql, NULL, array($_SERVER['REMOTE_USER'])));
-    } else {
-      $user = userFromSession();
-    }
+    } // end if empty($user)
 
     if (!empty($user)) {
       // generate it once here, while session is open.  Value will be cached in session and return when called later on
