@@ -42,6 +42,7 @@ if (
   $protocol = 'http';
 }
 define('ZM_BASE_PROTOCOL', $protocol);
+define('DIR_EXPORTS_DOWNLOAD', ZM_DIR_EXPORTS.'/files_for_download');
 
 // Absolute URL's are unnecessary and break compatibility with reverse proxies 
 // define( "ZM_BASE_URL", $protocol.'://'.$_SERVER['HTTP_HOST'] );
@@ -196,6 +197,11 @@ require_once('includes/auth.php');
 # Only one request can open the session file at a time, so let's close the session here to improve concurrency.
 # Any file/page that sets session variables must re-open it.
 session_write_close();
+if ($redirect) {
+  ZM\Debug("Redirecting to $redirect");
+  header('Location: '.$redirect);
+  return;
+}
 
 require_once('includes/Storage.php');
 require_once('includes/Event.php');
@@ -249,9 +255,10 @@ if ( ZM_OPT_USE_AUTH and (!isset($user)) and ($view != 'login') and ($view != 'n
     exit;
   }
   $view = 'none';
-  $redirect = '?view=login';
+  $postLoginQuery = $_SERVER['QUERY_STRING'];
+  $redirect = '?view=login&postLoginQuery=' . urlencode($postLoginQuery);
   zm_session_start();
-  $_SESSION['postLoginQuery'] = $_SERVER['QUERY_STRING'];
+  $_SESSION['postLoginQuery'] = $postLoginQuery;
   session_write_close();
   ZM\Debug("Redirecting to $redirect");
   header('Location: '.$redirect);
