@@ -237,13 +237,14 @@ export class VideoRTC extends HTMLElement {
      */
     oninit() {
         this.video = document.createElement('video');
-        this.video.controls = true;
+        this.video.controls = false;
         this.video.playsInline = true;
         this.video.preload = 'auto';
 
         this.video.style.display = 'block'; // fix bottom margin 4px
         this.video.style.width = '100%';
         this.video.style.height = '100%';
+        this.video.muted = this.muted;
 
         this.appendChild(this.video);
 
@@ -334,8 +335,10 @@ export class VideoRTC extends HTMLElement {
                 for (const mode in this.onmessage) {
                     this.onmessage[mode](msg);
                 }
-            } else {
+            } else if (this.ondata) {
                 this.ondata(ev.data);
+            } else {
+              console.log('No ondata to handle', ev);
             }
         });
 
