@@ -963,10 +963,10 @@ Alltid: En ZMC prosess vil kjøre og umiddelbart koble til og forbli tilkoblet.~
       Manuell setting av Profil-ID, som kan tvinge en nettleser til å forsøke å spille en gitt strøm. Forsøk "42e01f"
       for en universell støttet verdi, eller la stå blank for å bruke Profil-IDIen oppgitt av kilden.'
   ),
-  'FUNCTION_JANUS_USE_RTSP_RESTREAM' => array(
+  'FUNCTION_RESTREAM' => array(
     'Hjelp' => '
-      Hvis kameraet ikke fungerer under Janus uten tillegsinstillinger, aktiver denne for å bruke ZoneMinder
-      RTSP restream som Janus kilde.'
+      Aktiver denne for å bruke ZoneMinder RTSP restream som kilde for strømmetjenester
+      (Janus, Go2RTC, RTSP2Web) i stedet for kameraet direkte. Nyttig hvis kameraet ikke fungerer med en strømmetjeneste direkte.'
   ),
   'FUNCTION_JANUS_RTSP_SESSION_TIMEOUT' => array(
     'Hjelp' => '
