@@ -85,7 +85,7 @@ function ajaxRequest(params) {
     }
 
     // Iterate over form elements instead of using serialize
-    for (var i = 0; i < form.elements.length; i++) {
+    for (let i = 0; i < form.elements.length; i++) {
       var element = form.elements[i];
       var name = element.name;
 
@@ -100,7 +100,7 @@ function ajaxRequest(params) {
       if (element.type === 'select-multiple' || (element.multiple && element.tagName === 'SELECT')) {
         // Handle multi-select dropdowns
         var selectedOptions = [];
-        for (var j = 0; j < element.options.length; j++) {
+        for (let j = 0; j < element.options.length; j++) {
           if (element.options[j].selected) {
             selectedOptions.push(element.options[j].value);
           }
@@ -497,21 +497,6 @@ function initPage() {
   if (consoleRefreshTimeout > 0) {
     setInterval(reloadWindow, consoleRefreshTimeout);
   }
-
-  if ( showDonatePopup ) {
-    $j.getJSON(thisUrl + '?request=modal&modal=donate')
-        .done(function(data) {
-          insertModalHtml('donate', data.html);
-          $j('#donate').modal('show');
-          // Manage the Apply button
-          $j('#donateApplyBtn').click(function(evt) {
-            evt.preventDefault();
-            $j('#donateForm').submit();
-          });
-        })
-        .fail(logAjaxFail);
-  }
-
   // Setup the thumbnail video animation after table loads
   table.on('post-body.bs.table', function() {
     if (!isMobile()) initThumbAnimation();
