@@ -197,12 +197,6 @@ function deleteFilter(element) {
   }
 }
 
-var escape = document.createElement('textarea');
-function escapeHTML(html) {
-  escape.textContent = html;
-  return escape.innerHTML;
-}
-
 function parseRows(rows) {
   for ( let rowNum = 0; rowNum < rows.length; rowNum++ ) { //Each row is a term
     const queryPrefix = 'filter[Query][terms][';
@@ -222,8 +216,8 @@ function parseRows(rows) {
     if ( brackets > 0 ) { // add bracket select to all rows
       const obrSelect = $j('<select></select>').attr('name', queryPrefix + rowNum + '][obr]').attr('id', queryPrefix + rowNum + '][obr]');
       const cbrSelect = $j('<select></select>').attr('name', queryPrefix + rowNum + '][cbr]').attr('id', queryPrefix + rowNum + '][cbr]');
-      obrSelect.addClass('chosen').attr('data-placeholder', ' ').append('<option value="0"</option>');
-      cbrSelect.addClass('chosen').attr('data-placeholder', ' ').append('<option value="0"</option>');
+      obrSelect.addClass('chosen').attr('data-placeholder', ' ').append('<option value="0"></option>');
+      cbrSelect.addClass('chosen').attr('data-placeholder', ' ').append('<option value="0"></option>');
       for ( let i = 1; i <= brackets; i++ ) { // build bracket options
         obrSelect.append('<option value="' + i + '">' + '('.repeat(i) + '</option>');
         cbrSelect.append('<option value="' + i + '">' + ')'.repeat(i) + '</option>');
@@ -255,8 +249,8 @@ function parseRows(rows) {
       inputTds.eq(4).html(archiveSelect).children().val(archiveVal).addClass('chosen chosen-full-width');
     } else if ( attr == 'AlarmedZoneId' ) {
       const zoneSelect = $j('<select></select>').attr('name', queryPrefix + rowNum + '][val]').attr('id', queryPrefix + rowNum + '][val]');
-      for ( monitor_id in monitors ) {
-        for ( zone_id in zones ) {
+      for ( const monitor_id in monitors ) {
+        for ( const zone_id in zones ) {
           const zone = zones[zone_id];
           if ( monitor_id == zone.MonitorId ) {
             zoneSelect.append('<option value="' + zone_id + '">' + zone.Name + '</option>');
