@@ -3,6 +3,7 @@ const monitors = new Array();
 var monitors_ul = null;
 var idleTimeoutTriggered = false; /* Timer ZM_WEB_VIEWING_TIMEOUT has been triggered */
 var monitorInitComplete = false;
+var resizeInterval = null;
 
 const VIEWING = 0;
 const EDITING = 1;
@@ -319,6 +320,7 @@ function setRatioForMonitor(objStream, id=null) {
   if (!height) {
     console.log("0 height from ", currentMonitor.width, currentMonitor.height, (currentMonitor.width / currentMonitor.height > 1), objStream.clientWidth / ratio);
   } else {
+    //console.log('good height', height, objStream);
     objStream.style['height'] = (objStream.naturalHeight === undefined || objStream.naturalHeight > 20) ? height + 'px' : 'auto';
     objStream.parentNode.style['height'] = height + 'px';
   }
@@ -481,6 +483,7 @@ function startMonitors() {
     const monitor = monitors[i];
     const isOut = isOutOfViewport(monitor.getElement());
     if (!isOut.all) {
+      monitor.setPlayer(monitor.player);
       monitor.start();
     }
     if ((monitor.type == 'WebSite') && (monitor.refresh > 0)) {
@@ -551,7 +554,7 @@ function fullscreenchanged(event) {
 function calculateAverageMonitorsRatio(arrRatioMonitors) {
   //Let's calculate the average Ratio value for the displayed monitors
   let total = 0;
-  for (var i = 0; i < arrRatioMonitors.length; i++) {
+  for (let i = 0; i < arrRatioMonitors.length; i++) {
     total += arrRatioMonitors[i];
   }
   const avg = total / arrRatioMonitors.length;
@@ -591,40 +594,30 @@ function initPage() {
   //  $j('#zmMontageLayout').val(getCookie('zmMontageLayout'));
   //}
 
-  document.querySelectorAll(".monitorStream, .ratioControl").forEach(function(el) {
+  document.querySelectorAll(".grid-monitor").forEach(function(el) {
     // Displaying & hiding "Scale" and other buttons, at the top of the monitor image,  monitor status information
     el.addEventListener('mouseout', function addListenerMouseover(event) {
       const id = stringToNumber(el.id);
-      if (event.target && event.relatedTarget) {
-        if (event.relatedTarget.closest('#imageFeed'+id) && event.target.closest('#imageFeed'+id)) {
-          return;
+      if (!(event.target && event.target.closest('#m'+id) && event.relatedTarget && event.relatedTarget.closest('#m'+id))) { // This will prevent the code below from being executed if we navigate within a single monitor block.
+        if (!event.relatedTarget || (event.relatedTarget && !event.relatedTarget.closest('#m'+id))) {
+          if ($j('#monitorStatusPosition').val() == 'showOnHover') {
+            $j('#monitors').find('#monitorStatus'+id).addClass('hidden');
+          }
+          hideСontrolElementsOnStream(el);
         }
       }
-      if (!event.relatedTarget) {
-        hideСontrolElementsOnStream(el);
-        return;
-      }
-      if (!event.relatedTarget.closest('#imageFeed'+id) && (!event.relatedTarget.closest('#ratioControl'+id))) {
-        if ($j('#monitorStatusPosition').val() == 'showOnHover') {
-          $j('#monitors').find('#monitorStatus'+id).addClass('hidden');
-        }
-        hideСontrolElementsOnStream(el);
-      }
     });
 
     el.addEventListener('mouseover', function addListenerMouseover(event) {
       const id = stringToNumber(el.id);
-      if (event.target && event.relatedTarget) {
-        if (event.relatedTarget.closest('#imageFeed'+id) && event.target.closest('#imageFeed'+id)) {
-          return;
+      if (!(event.target && event.target.closest('#m'+id) && event.relatedTarget && event.relatedTarget.closest('#m'+id))) {
+        if (event.target.closest('#m'+id)) {
+          if ($j('#monitorStatusPosition').val() == 'showOnHover') {
+            $j('#monitors').find('#monitorStatus'+id).removeClass('hidden');
+          }
+          showСontrolElementsOnStream(el);
         }
       }
-      if (event.target.closest('#imageFeed'+id) || event.target.closest('#ratioControl'+id)) {
-        if ($j('#monitorStatusPosition').val() == 'showOnHover') {
-          $j('#monitors').find('#monitorStatus'+id).removeClass('hidden');
-        }
-        showСontrolElementsOnStream(el);
-      }
     });
   });
 
@@ -676,6 +669,7 @@ function initPage() {
                   for (let i=0, length = monitors.length; i < length; i++) {
                     const monitor = monitors[i];
                     if ((!isOutOfViewport(monitor.getElement()).all) && !monitor.started) {
+                      monitor.setPlayer(monitor.player);
                       monitor.start();
                     }
                   }
@@ -696,7 +690,7 @@ function initPage() {
     inactivityTime();
   }
 
-  setInterval(() => { //Updating GridStack resizeToContent, Scale & Ratio
+  resizeInterval = setInterval(() => { //Updating GridStack resizeToContent, Scale & Ratio
     if (changedMonitors.length > 0) {
       changedMonitors.slice().reverse().forEach(function(item, index, object) {
         const img = getStream(item);
@@ -756,15 +750,17 @@ function showСontrolElementsOnStream(stream) {
 
 function on_scroll() {
   if (!monitorInitComplete || idleTimeoutTriggered) return;
+
   for (let i = 0, length = monitors.length; i < length; i++) {
     const monitor = monitors[i];
 
     const isOut = isOutOfViewport(monitor.getElement());
+    //console.log(isOut, monitor.id);
     if (!isOut.all) {
       if (!monitor.started) monitor.start();
     } else if (monitor.started) {
-      //monitor.stop(); // does not work without replacing SRC to stop ZMS
-      monitor.kill();
+      monitor.stop(); // does not work without replacing SRC to stop ZMS
+      //monitor.kill();
     }
   } // end foreach monitor
 } // end function on_scsroll
@@ -793,6 +789,7 @@ function isOutOfViewport(elem) {
 };
 
 function formSubmit(form) {
+  clearInterval(resizeInterval);
   console.log("Killing streaming");
   for (let i = 0, length = monitors.length; i < length; i++) {
     if (monitors[i]) {
@@ -877,6 +874,7 @@ function monitorsSetScale(id=null) {
     }
     const el = document.getElementById('liveStream'+id);
     const panZoomScale = (panZoomEnabled && zmPanZoom.panZoom[id] ) ? zmPanZoom.panZoom[id].getScale() : 1;
+    console.log("monitorsSetsCale", id, 'clientWidth', el.clientWidth, 'clientHeight', el.clientHeight, 'panzoomscale', panZoomScale);
     currentMonitor.setScale(0, el.clientWidth * panZoomScale + 'px', el.clientHeight * panZoomScale + 'px', {resizeImg: false, streamQuality: $j('#streamQuality').val()});
   } else {
     for ( let i = 0, length = monitors.length; i < length; i++ ) {
@@ -1047,9 +1045,10 @@ document.onvisibilitychange = () => {
   if (document.visibilityState === "hidden") {
     TimerHideShow = clearTimeout(TimerHideShow);
     TimerHideShow = setTimeout(function() {
-      //Stop monitors when closing or hiding page
-      //closing should kill, hiding should stop
+      //Stop monitors when hiding page
+      //closing should kill, hiding should stop/pause
       for (let i = 0, length = monitors.length; i < length; i++) {
+        // Stop instead of pause because we don't want buffering in zms
         monitors[i].stop();
       }
     }, 15*1000);
@@ -1062,6 +1061,7 @@ document.onvisibilitychange = () => {
 
         const isOut = isOutOfViewport(monitor.getElement());
         if ((!isOut.all) && !monitor.started) {
+          monitor.setPlayer(monitor.player);
           monitor.start();
         }
       } // end foreach monitor
@@ -1072,6 +1072,7 @@ document.onvisibilitychange = () => {
 
 window.onbeforeunload = function(e) {
   console.log('unload');
+  clearInterval(resizeInterval);
   /*
   //event.preventDefault();
   for (let i = 0, length = monitorData.length; i < length; i++) {
