@@ -61,18 +61,21 @@ function evaluateLoadTimes() {
   }
   avgFrac = avgFrac / imageLoadTimesEvaluated;
   // The larger this is (positive) the faster we can go
-  if (avgFrac >= 0.9) currentDisplayInterval = (currentDisplayInterval * 0.50).toFixed(1); // we can go much faster
-  else if (avgFrac >= 0.8) currentDisplayInterval = (currentDisplayInterval * 0.55).toFixed(1);
-  else if (avgFrac >= 0.7) currentDisplayInterval = (currentDisplayInterval * 0.60).toFixed(1);
-  else if (avgFrac >= 0.6) currentDisplayInterval = (currentDisplayInterval * 0.65).toFixed(1);
-  else if (avgFrac >= 0.5) currentDisplayInterval = (currentDisplayInterval * 0.70).toFixed(1);
-  else if (avgFrac >= 0.4) currentDisplayInterval = (currentDisplayInterval * 0.80).toFixed(1);
-  else if (avgFrac >= 0.35) currentDisplayInterval = (currentDisplayInterval * 0.90).toFixed(1);
-  else if (avgFrac >= 0.3) currentDisplayInterval = (currentDisplayInterval * 1.00).toFixed(1);
-  else if (avgFrac >= 0.25) currentDisplayInterval = (currentDisplayInterval * 1.20).toFixed(1);
-  else if (avgFrac >= 0.2) currentDisplayInterval = (currentDisplayInterval * 1.50).toFixed(1);
-  else if (avgFrac >= 0.1) currentDisplayInterval = (currentDisplayInterval * 2.00).toFixed(1);
-  else currentDisplayInterval = (currentDisplayInterval * 2.50).toFixed(1);
+  // Note: multiply then round to 1 decimal place, keeping result as number
+  let multiplier;
+  if (avgFrac >= 0.9) multiplier = 0.50; // we can go much faster
+  else if (avgFrac >= 0.8) multiplier = 0.55;
+  else if (avgFrac >= 0.7) multiplier = 0.60;
+  else if (avgFrac >= 0.6) multiplier = 0.65;
+  else if (avgFrac >= 0.5) multiplier = 0.70;
+  else if (avgFrac >= 0.4) multiplier = 0.80;
+  else if (avgFrac >= 0.35) multiplier = 0.90;
+  else if (avgFrac >= 0.3) multiplier = 1.00;
+  else if (avgFrac >= 0.25) multiplier = 1.20;
+  else if (avgFrac >= 0.2) multiplier = 1.50;
+  else if (avgFrac >= 0.1) multiplier = 2.00;
+  else multiplier = 2.50;
+  currentDisplayInterval = Math.round(currentDisplayInterval * multiplier * 10) / 10;
   // limit this from about 40fps to .1 fps
   currentDisplayInterval = Math.min(Math.max(currentDisplayInterval, 40), 10000);
   imageLoadTimesEvaluated=0;
@@ -373,10 +376,10 @@ function getImageSource(monId, time) {
     const server = storage.ServerId ? Servers[storage.ServerId] : Servers[monitorServerId[monId]];
     return server.PathToZMS + '?' +
     //mode=jpeg
-     "mode=single"+
+      "mode=single" +
       "&event=" + Frame.EventId +
-		  //'&frame='+frame_id +
-		  '&time='+time +
+      //'&frame='+frame_id +
+      '&time='+time +
       //"&width=" + monitorCanvasObj[monId].width +
       //"&height=" + monitorCanvasObj[monId].height +
       "&scale=" + scale +
@@ -603,6 +606,11 @@ function drawEventOnGraph(zm_event) {
 }
 
 function drawGraph() {
+  if (!canvas) {
+    // Likely live mode
+    console.log("Called drawGraph while in live mode");
+    return;
+  }
   underSlider = undefined; // flag we don't have a slider cached
 
   // timelinediv starts off 100% of browser, but it's container can be smaller
@@ -863,7 +871,7 @@ function showScale(newscale) {
 function setScale(newscale) {
   // makes actual change
   showScale(newscale);
-  for ( var i=0; i < numMonitors; i++ ) {
+  for ( let i=0; i < numMonitors; i++ ) {
     monitorCanvasObj[monitorPtr[i]].width = monitorWidth[monitorPtr[i]]*monitorNormalizeScale[monitorPtr[i]]*monitorZoomScale[monitorPtr[i]]*newscale;
     monitorCanvasObj[monitorPtr[i]].height = monitorHeight[monitorPtr[i]]*monitorNormalizeScale[monitorPtr[i]]*monitorZoomScale[monitorPtr[i]]*newscale;
   }
@@ -940,7 +948,7 @@ function clicknav(minSecs, maxSecs, live) {// we use the current time if we can
   }
 
   var zoomStr = "";
-  for ( var i = 0; i < numMonitors; i++ ) {
+  for ( let i = 0; i < numMonitors; i++ ) {
     if ( monitorZoomScale[monitorPtr[i]] < 0.99 || monitorZoomScale[monitorPtr[i]] > 1.01 ) { // allow for some up/down changes and just treat as 1 of almost 1
       zoomStr += "&z" + monitorPtr[i].toString() + "=" + monitorZoomScale[monitorPtr[i]].toFixed(2);
     }
@@ -1159,29 +1167,12 @@ function loadEventData(e) {
     mon_ids[mon_ids.length] = monitor.Id;
   }
 
-  var url = Servers[serverId].urlToApi()+'/events/index';
   $j('#fieldsTable input,#fieldsTable select').each(function(index) {
     const el = $j(this);
     const val = el.val();
     if (val && (!Array.isArray(val) || val.length)) {
       const name = el.attr('name');
-
       if (name) {
-        const found = name.match(/filter\[Query\]\[terms\]\[(\d)+\]\[val\]/);
-        if (found) {
-          const attr_name = 'filter[Query][terms]['+found[1]+'][attr]';
-          const attr = this.form.elements[attr_name];
-          const op_name = 'filter[Query][terms]['+found[1]+'][op]';
-          const op = this.form.elements[op_name];
-          if (attr) {
-            if (attr.value==='Monitor') attr.value='MonitorId';
-            url += '/'+attr.value+' '+op.value+':'+encodeURIComponent(val);
-          } else {
-            console.log('No attr for '+attr_name);
-          }
-        //} else {
-          //console.log("No match for " + name);
-        }
         data[name] = val;
         const cookie = el.attr('data-cookie');
         if (cookie) setCookie(cookie, val, 3600);
@@ -1194,17 +1185,17 @@ function loadEventData(e) {
       alert(data.message);
       return;
     }
-    if (!data.events) {
+    if (!data.rows) {
       console.log(data);
       return;
     }
     console.log("Event data ", data);
 
-    if (data.events.length) {
+    if (data.rows.length) {
       // event_list is solely for sending to loadFrames
       const event_list = {};
-      for (let i=0, len = data.events.length; i<len; i++) {
-        const ev = data.events[i].Event;
+      for (let i=0, len = data.rows.length; i<len; i++) {
+        const ev = data.rows[i];
         ev.Id = parseInt(ev.Id);
         ev.MonitorId = parseInt(ev.MonitorId);
         event_list[ev.Id] = events[ev.Id] = ev;
@@ -1212,41 +1203,29 @@ function loadEventData(e) {
         if ((!(ev.MonitorId in events_for_monitor)) || !events_for_monitor[ev.MonitorId]) {
           events_for_monitor[ev.MonitorId] = []; // id=>event
         }
-        //events_by_monitor_id[ev.MonitorId].push(ev.Id);
         events_for_monitor[ev.MonitorId].push(ev);
-        //drawEventOnGraph(ev);
       }
       loadFrames(event_list).then(function() {
-        console.log("have ffaremes, drawing graph");
+        console.log("have frames, drawing graph");
         drawGraph();
-        /*
-        // HACK to refresh monitor names over event data
-        for (let i=0; i < numMonitors; i++) {
-          // Apparently we have to set these each time before calling fillText
-          ctx.font = parseInt(rowHeight * timeLabelsFractOfRow).toString() + "px Georgia";
-          ctx.globalAlpha = 1;
-          ctx.fillStyle = "white";
-          // This should roughly center font in row
-          ctx.fillText(monitorName[monitorPtr[i]], 0, (i + 1 - (1 - timeLabelsFractOfRow)/2 ) * rowHeight);
-        }
-        //underSlider = ctx.getImageData(sliderX, 0, sliderWidth, sliderHeight);
-        */
       });
     } else {
       console.log("No events in data?");
     }
   } // end function receive_events
 
-  //FIXME ajax gets overwrritten by subsequent monitor
+  //FIXME ajax gets overwritten by subsequent monitor
   if (ajax) ajax.abort();
 
   if (mon_ids.length) {
     for (let i=0; i < mon_ids.length; i++) {
+      // Add MonitorId filter to data for this specific monitor
+      const monitor_data = Object.assign({}, data);
+      monitor_data['MonitorId'] = mon_ids[i];
       ajax = $j.ajax({
-        url: url+ '/MonitorId:'+mon_ids[i]+ '.json'+'?'+auth_relay,
-        method: 'GET',
-        //url: thisUrl + '?view=request&request=events&task=query&sort=Id&order=ASC',
-        //data: data,
+        url: thisUrl + '?view=request&request=events&task=query&sort=Id&order=ASC',
+        method: 'POST',
+        data: monitor_data,
         timeout: 0,
         success: receive_events,
         error: function(jqXHR) {
@@ -1257,10 +1236,9 @@ function loadEventData(e) {
     } // end foreach monitor
   } else {
     ajax = $j.ajax({
-      url: url+'.json'+'?'+auth_relay,
-      method: 'GET',
-      //url: thisUrl + '?view=request&request=events&task=query&sort=Id&order=ASC',
-      //data: data,
+      url: thisUrl + '?view=request&request=events&task=query&sort=Id&order=ASC',
+      method: 'POST',
+      data: data,
       timeout: 0,
       success: receive_events,
       error: function(jqXHR) {
@@ -1313,9 +1291,8 @@ function initPage() {
       alert("Couldn't find DOM element for Monitor" + monId + "monitorPtr.length=" + len);
       continue;
     }
-    console.log("Setting up imagedone for ", monId);
-    monitorCanvasCtx[monId] = monitorCanvasObj[monId].getContext('2d');
-
+    canvasObj.addEventListener('click', clickMonitor, false);
+    monitorCanvasCtx[monId] = canvasObj.getContext('2d');
     const imageObject = monitorImageObject[monId] = new Image();
     imageObject.monId = monId;
     imageObject.onload = function() {
@@ -1327,7 +1304,6 @@ function initPage() {
     if (liveMode) {
       loadImage2Monitor(monId, monitorImageURL[monId]);
     }
-    canvasObj.addEventListener('click', clickMonitor, false);
   } // end foreach monitor
 
   setSpeed(speedIndex);
@@ -1394,7 +1370,7 @@ function wait_for_events() {
 }
 
 function takeSnapshot() {
-  monitor_ids = [];
+  const monitor_ids = [];
   for (const key in monitorIndex) {
     monitor_ids[monitor_ids.length] = key;
   }
@@ -1479,8 +1455,8 @@ function loadFrames(zm_events) {
             } // end if there are frames
             resolve();
           },
-          error: function() {
-            logAjaxFail;
+          error: function(jqXHR) {
+            logAjaxFail(jqXHR);
             reject(Error("There was an error"));
           }
         }); // end ajax
