@@ -45,7 +45,9 @@ monitorData[monitorData.length] = {
   'DefaultPlayer':'<?php echo $m->DefaultPlayer() ?>',
   'StreamChannel':'<?php echo $m->StreamChannel() ?>',
   'Go2RTCEnabled': <?php echo $m->Go2RTCEnabled() ?>,
-  'SecondPath':'<?php echo $m->SecondPath() ?>',
+  'Path':'<?php echo validJsStr($m->Path()) ?>',
+  'SecondPath':'<?php echo validJsStr($m->SecondPath()) ?>',
+  'Restream':<?php echo $m->Restream() ? 'true' : 'false' ?>,
   'janusEnabled':<?php echo $m->JanusEnabled() ?>,
   'url': '<?php echo $m->UrlToIndex(ZM_MIN_STREAMING_PORT ? ($m->Id() + ZM_MIN_STREAMING_PORT) : '') ?>',
   'url_to_zms': '<?php echo $m->UrlToZMS( ZM_MIN_STREAMING_PORT ? ($m->Id() + ZM_MIN_STREAMING_PORT) : '') ?>',
