@@ -71,8 +71,8 @@ echo getNavBarHTML();
           </div>
         </div><!--header-->
         <div id="content-inner">
-          <div class="row">
-            <div class="col-lg-6">
+          <div class="BasicInformation row">
+            <div class="col-lg-4">
               <fieldset>
                 <legend><?php echo translate('Account Details') ?></legend>
                 <table class="table table-sm">
@@ -121,7 +121,7 @@ echo getNavBarHTML();
                 </table>
               </fieldset>
             </div>
-            <div class="col-lg-6">
+            <div class="col-lg-4">
               <fieldset>
                 <legend><?php echo translate('Preferences') ?></legend>
                 <table class="table table-sm">
@@ -182,9 +182,9 @@ echo htmlSelect('user[HomeView]', $homeview_options, $User->HomeView(), ['class'
                 </table>
               </fieldset>
             </div>
-          </div><!--end row-->
 
-<?php if (ZM_OPT_USE_API && $User->Id() && $User->APIEnabled()) {
+<?php
+if (ZM_OPT_USE_API && $User->Id() && $User->APIEnabled()) {
   // Determine profile name: ZM_WEB_TITLE if not default, else ZM_HOME_URL if not default, else 'ZoneMinder'
   $profileName = 'ZoneMinder';
   if (defined('ZM_WEB_TITLE') && ZM_WEB_TITLE !== 'ZoneMinder' && ZM_WEB_TITLE !== '') {
@@ -192,34 +192,33 @@ echo htmlSelect('user[HomeView]', $homeview_options, $User->HomeView(), ['class'
   } else if (defined('ZM_HOME_URL') && ZM_HOME_URL !== 'https://zoneminder.com' && ZM_HOME_URL !== '') {
     $profileName = ZM_HOME_URL;
   }
-
   // Construct portal URL from current request
   $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') ? 'https' : 'http';
   $host = $_SERVER['HTTP_HOST'] ?? $_SERVER['SERVER_NAME'] ?? 'localhost';
   $path = dirname($_SERVER['SCRIPT_NAME']);
   $portalUrl = $protocol . '://' . $host . $path;
 ?>
-        <div class="zmNgOnboarding mt-4">
-          <fieldset>
-            <legend><?php echo translate('zmNg Mobile App QR Code') ?></legend>
-            <div id="zmNgQRSection" class="text-center py-3">
-              <div id="zmNgQRCode" style="padding: 15px; background: white; display: inline-block; border-radius: 8px; box-shadow: 0 2px 8px rgba(0,0,0,0.1);"></div>
-              <div id="zmNgQRMessage" class="text-muted mt-2 mb-3">
-                <small><?php echo translate('Scan this QR code with the zmNg app to add this profile') ?></small>
-              </div>
-              <div class="form-inline justify-content-center">
-                <label for="zmNgPassword" class="mr-2"><?php echo translate('Password') ?>:</label>
-                <div class="input-group input-group-sm" style="width: 200px;">
-                  <input type="password" class="form-control" id="zmNgPassword" placeholder="<?php echo translate('Optional') ?>" autocomplete="off"/>
-                  <div class="input-group-append">
-                    <span class="input-group-text material-icons md-18" style="cursor: pointer;" data-on-click-this="toggle_password_visibility" data-password-input="zmNgPassword">visibility</span>
+            <div class="col-lg-4 zmNgOnboarding">
+              <fieldset>
+                <legend><?php echo translate('zmNg Mobile App QR Code') ?></legend>
+                <div id="zmNgQRSection" class="text-center py-3">
+                  <div id="zmNgQRCode" style="padding: 15px; background: white; display: inline-block; border-radius: 8px; box-shadow: 0 2px 8px rgba(0,0,0,0.1);"></div>
+                  <div id="zmNgQRMessage" class="text-muted mt-2 mb-3">
+                    <small><?php echo translate('Scan this QR code with the zmNg app to add this profile') ?></small>
                   </div>
+                  <div class="form-inline justify-content-center">
+                    <label for="zmNgPassword" class="mr-2"><?php echo translate('Password') ?>:</label>
+                    <div class="input-group input-group-sm" style="width: 200px;">
+                      <input type="password" class="form-control" id="zmNgPassword" placeholder="<?php echo translate('Optional') ?>" autocomplete="off"/>
+                      <div class="input-group-append">
+                        <span class="input-group-text material-icons md-18" style="cursor: pointer;" data-on-click-this="toggle_password_visibility" data-password-input="zmNgPassword">visibility</span>
+                      </div>
+                    </div>
+                  </div>
+                  <div class="text-muted mt-2"><small><?php echo translate('Enter password to include it in the QR code') ?></small></div>
                 </div>
-              </div>
-              <div class="text-muted mt-2"><small><?php echo translate('Enter password to include it in the QR code') ?></small></div>
+              </fieldset>
             </div>
-          </fieldset>
-        </div>
         <?php echo output_script('js/qrcode.min.js'); ?>
         <script nonce="<?php echo $cspNonce ?>">
         document.addEventListener('DOMContentLoaded', function() {
@@ -265,16 +264,43 @@ echo htmlSelect('user[HomeView]', $homeview_options, $User->HomeView(), ['class'
           passwordInput.addEventListener('input', generateQRCode);
         });
         </script>
-<?php } ?>
-
+<?php
+} # end if API
+?>
+          </div><!--end row-->
 <?php
 if (canEdit('System')) {
 ?>
-        <div class="Permissions mt-4">
-          <fieldset>
-            <legend><?php echo translate('Global Permissions') ?></legend>
-            <div class="row">
-              <div class="col-md-6">
+          <div class="Permissions row">
+            <div id="ViewPermissions" class="col-lg-6">
+              <fieldset><legend><?php echo translate('View Permissions') ?></legend>
+            <table id="viewPermissionsTable" class="major">
+              <thead class="thead-highlight">
+                <th class="View"><?php echo translate('View') ?></th>
+                <th class="permission"><?php echo translate('Permission') ?></th>
+              </thead>
+              <tbody>
+        <?php
+          global $available_views;
+          $views = array_keys($available_views);
+          sort($views);
+          foreach ($views as $view) {
+            echo '
+          <tr class="view">
+            <td class="View">'.validHtmlStr($available_views[$view]).'</td>
+            <td class="permission">'.html_radio('view_permissions['.$view.']', $yesno,
+              $User->View_Permission($view)->Permission(),
+            ).'</td>
+          </tr>';
+          }
+        ?>
+              </tbody>
+            </table>
+          </fieldset>
+              </div><!--View Permissions-->
+            <div class="GlobalPermissions col-lg-6">
+              <fieldset>
+                <legend><?php echo translate('Global Permissions') ?></legend>
                 <table class="table table-sm">
                   <tbody>
                     <tr>
@@ -295,12 +321,6 @@ if (canEdit('System')) {
                       <th scope="row"><?php echo translate('Control') ?></th>
                       <td><?php echo htmlSelect('user[Control]', $nve, $User->Control(), ['class'=>'form-control form-control-sm']) ?></td>
                     </tr>
-                  </tbody>
-                </table>
-              </div>
-              <div class="col-md-6">
-                <table class="table table-sm">
-                  <tbody>
                     <tr>
                       <th scope="row"><?php echo translate('Monitors') ?></th>
                       <td><?php echo htmlSelect('user[Monitors]', $nvec, $User->Monitors(), ['id'=>'user[Monitors]', 'class'=>'form-control form-control-sm', 'data-on-change'=>'updateEffectivePermissions']) ?></td>
@@ -319,10 +339,10 @@ if (canEdit('System')) {
                     </tr>
                   </tbody>
                 </table>
-              </div>
-            </div>
-          </fieldset>
-        </div><!--Permissions-->
+              </fieldset>
+            </div><!--GlobalPermissions-->
+          </div><!--Permissions-->
+          <div class="row mt-4">
 <?php
 $groups = array();
 if (canEdit('Groups')) {
@@ -341,18 +361,18 @@ if (canEdit('Groups')) {
       $max_depth = $group->depth();
   }
 ?>
-        <div id="GroupPermissions" class="mt-4">
-          <fieldset>
-            <legend><?php echo translate('Groups Permissions') ?></legend>
-            <table class="table table-sm table-striped">
-              <thead class="thead-light">
-                <tr>
-                  <th class="name" colspan="<?php echo $max_depth+1 ?>"><?php echo translate('Name') ?></th>
-                  <th class="monitors"><?php echo translate('Monitors') ?></th>
-                  <th class="permission"><?php echo translate('Permission') ?></th>
-                </tr>
-              </thead>
-              <tbody>
+            <div id="GroupPermissions" class="col-lg-6">
+              <fieldset>
+                <legend><?php echo translate('Groups Permissions') ?></legend>
+                <table id="groupPermissionsTable" class="Groups table table-sm table-striped">
+                  <thead class="thead-light">
+                    <tr>
+                      <th class="name" colspan="<?php echo $max_depth+1 ?>"><?php echo translate('Name') ?></th>
+                      <th class="monitors"><?php echo translate('Monitors') ?></th>
+                      <th class="permission"><?php echo translate('Permission') ?></th>
+                    </tr>
+                  </thead>
+                  <tbody>
 <?php
   function group_line($group) {
     global $children;
@@ -382,49 +402,50 @@ if (canEdit('Groups')) {
     }
   }
 ?>
-              </tbody>
-            </table>
-          </fieldset>
-        </div><!--Group Permissions-->
+                  </tbody>
+                </table>
+              </fieldset>
+            </div><!--Group Permissions-->
 <?php
-  } // end if canEdit(Groups)
+} // end if canEdit(Groups)
 ?>
-        <div id="MonitorPermissions" class="mt-4">
-          <fieldset>
-            <legend><?php echo translate('Monitor Permissions') ?></legend>
-            <table class="table table-sm table-striped">
-              <thead class="thead-light">
-                <tr>
-                  <th class="Id"><?php echo translate('Id') ?></th>
-                  <th class="Name"><?php echo translate('Name') ?></th>
-                  <th class="permission"><?php echo translate('Permission') ?></th>
-                  <th class="effective_permission"><?php echo translate('Effective Permission') ?></th>
-                </tr>
-              </thead>
-              <tbody>
+            <div id="MonitorPermissions" class="col-lg-6">
+              <fieldset>
+                <legend><?php echo translate('Monitor Permissions') ?></legend>
+                <table id="monitorPermissionsTable" class="Monitors table table-sm table-striped">
+                  <thead class="thead-light">
+                    <tr>
+                      <th class="Id"><?php echo translate('Id') ?></th>
+                      <th class="Name"><?php echo translate('Name') ?></th>
+                      <th class="permission"><?php echo translate('Permission') ?></th>
+                      <th class="effective_permission"><?php echo translate('Effective Permission') ?></th>
+                    </tr>
+                  </thead>
+                  <tbody>
 <?php
   $monitors = ZM\Monitor::find(['Deleted'=>0], ['order'=>'Sequence ASC']);
   foreach ($monitors as $monitor) {
     if ($monitor->canView()) {
       echo '
-                <tr>
-                  <td class="Id">'.$monitor->Id().'</td>
-                  <td class="Name">'.validHtmlStr($monitor->Name()).'</td>
-                  <td class="permission">'.html_radio('monitor_permission['.$monitor->Id().']', $inve,
-                    $User->Monitor_Permission($monitor->Id())->Permission(),
-                    ['default'=>'Inherit'],
-                    ['data-on-change'=>'updateEffectivePermissions']).'</td>
-                  <td class="effective_permission" id="effective_permission'.$monitor->Id().'">'.translate($monitor->effectivePermission($User)).'</td>
-                </tr>';
+                    <tr>
+                      <td class="Id">'.$monitor->Id().'</td>
+                      <td class="Name">'.validHtmlStr($monitor->Name()).'</td>
+                      <td class="permission">'.html_radio('monitor_permission['.$monitor->Id().']', $inve,
+                        $User->Monitor_Permission($monitor->Id())->Permission(),
+                        ['default'=>'Inherit'],
+                        ['data-on-change'=>'updateEffectivePermissions']).'</td>
+                      <td class="effective_permission" id="effective_permission'.$monitor->Id().'">'.translate($monitor->effectivePermission($User)).'</td>
+                    </tr>';
     } else {
       ZM\Debug("Can't view monitor ".$monitor->Id(). ' ' .$monitor->canView());
     }
   }
 ?>
-              </tbody>
-            </table>
-          </fieldset>
-        </div><!--Monitor Permissions-->
+                  </tbody>
+                </table>
+              </fieldset>
+            </div><!--Monitor Permissions-->
+          </div><!--row Group/Monitor Permissions-->
 <?php
 } // end if canEdit(System)
 ?>
